#!/usr/bin/perl
#
use strict;
use warnings;
use Net::SNMP;

my $REVISION = "Revision 0.2";
my $PROGNAME = $0;
my $message;
my $ServerID;
my $SrvStatusControllers;
my $SrvStatusLogicalDrives;
my $SrvStatusPhysicalDrives;
my $SrvStatusStatusOverall;
my $SysStat;
my $SystemStatus;
my $SystemStatusDescr;
my $snmpresult;
my $noSession = "No SNMP-Session possible! Probably no SNMP-Agent installed.";
my $oidServerID = '.1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.7.1';
my $oidSysStat = '.1.3.6.1.4.1.231.2.10.2.11.2.1.0';
my $oidSrvStatusControllers = '.1.3.6.1.4.1.231.2.49.1.3.3.0';
my $oidSrvStatusLogicalDrives = '.1.3.6.1.4.1.231.2.49.1.3.1.0';
my $oidSrvStatusPhysicalDrives = '.1.3.6.1.4.1.231.2.49.1.3.2.0';
my $oidSrvStatusStatusOverall = '.1.3.6.1.4.1.231.2.49.1.3.4.0';
my $oidEnvTempPrefix = '.1.3.6.1.4.1.231.2.10.2.2.5.2.1.1.';
my $oidEnvTempStatus = $oidEnvTempPrefix . '3';
my $oidEnvTempWarn = $oidEnvTempPrefix . '6';
my $oidEnvTempCrit = $oidEnvTempPrefix . '8';
my $oidEnvTempValue = $oidEnvTempPrefix . '11';
my $oidEnvTempDesc = $oidEnvTempPrefix . '13';
my $snmptempresult;
my $PandoratempStatus;
my $oidEnvFanPrefix = '1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.';
my $oidEnvFanStatus = $oidEnvFanPrefix . '3';
my $oidEnvFanPurpose = $oidEnvFanPrefix . '4';
my $oidEnvFanSpeed = $oidEnvFanPrefix . '8';
my $oidEnvFanDesc = $oidEnvFanPrefix . '16';
my $fanStatus;
my $fanSpeed;
my $fanID;
my $fan_msg = '';
my $fanPurpose;
my $fanPurposeDescr;
my $snmpfanresult;
my $PandorafanStatus;
my $fanDesc;
my $fanStatusDescr;
my @SysStatDescr = (
	'dummy',
	'System Status OK (1)',
	'System Status Degraded (2)',
	'System Status Error (3)',
	'System Status Failed (4)',
	'System Status Unknown (5)'
);
my @col = ( 
	"$oidServerID", 
	"$oidSysStat", 
	"$oidSrvStatusControllers",
	"$oidSrvStatusLogicalDrives",
	"$oidSrvStatusPhysicalDrives",
	"$oidSrvStatusStatusOverall"
);
my @fanChecks = (
	$oidEnvFanStatus,
	$oidEnvFanPurpose,
	$oidEnvFanSpeed,
	$oidEnvFanDesc
);
my @tempChecks = (
	$oidEnvTempStatus,
	$oidEnvTempValue,
	$oidEnvTempWarn,
	$oidEnvTempCrit,
	$oidEnvTempDesc
);
my @tempStatus = (
    "dummy", "UNKNOWN", "not-available", "OK", "dummy", "FAILED",
    "WARNING", "CRITICAL"
);
my @temps = ();
my @fanStatusDesc = (
    "dummy", "UNKNOWN", "DISABLED", "OK", "FAIL", "prefailure-predicted",
    "redundant-fan-failed", "not-manageable", "not-present"
);
my @fanPurposeDesc = (
    "dummy",     "UNKNOWN", "DISABLED",      "cpu-onchip",
    "component", "housing", "power-supply", "not-available"
);
my @fans = ();

sub print_usage {
    print "
    Usage: $PROGNAME <host> <community>
    \n";
}

sub print_revision() {
    print "
    $PROGNAME  - $REVISION
    Author: Geert Bracke
    2011-09-28
    Licence: GPL - http://www.fsf.org/licenses/gpl.txt
    \n";
}

sub print_help() {
    print_revision();
    print_usage();
}

sub main() {

    # SNMPv1 Login
    my ( $session, $error ) = Net::SNMP->session(
        -hostname  => $ARGV[0],
        -community => $ARGV[1],
        -port      => 161
    );

    if ( !defined $session ) {
        print "$noSession\n";
        exit 3;
    }
    #retrieve single OID's 
    $snmpresult = $session->get_request( @col );
    if ( !defined $snmpresult ) {
        $message = $session->error;
        $session->close;
        print "SNMP-Agent returned \"$message\". (Is the Serverview Agent installed on the checked system?)\n";
        exit 3;
    }

    #Resolve ServerID
    $ServerID = $$snmpresult{"$oidServerID"};
    #Print the Server ID
    print "<module>\n";
    print "<name>Fujitsu_ServerID</name>\n";
    print "<type>generic_data_string</type>\n";
    print "<data><![CDATA[" . $ServerID . "]]>\n</data>\n";
    print "<description><![CDATA[Fujitsu Server ID]]>\n</description>\n";
    print "</module>\n";

    #Resolve SystemStatus
    $SysStat = $$snmpresult{"$oidSysStat"};
    #  We evaluate the SysStat and generate a "Pandora status"
    if ($SysStat == 1)					#Ok status
    {
 	    $SystemStatus = "1";
    }
    else						#Degraded, error, failed, unknown
    {
	    $SystemStatus = "0";
    }

    # We evaluate the correct message to show the SystemStatus description
    $SystemStatusDescr = $SysStatDescr[$SysStat];

    print "<module>\n";
    print "<name>Fujitsu_SystemStatus</name>\n";
    print "<type>generic_proc</type>\n";
    print "<data>" . $SystemStatus . "</data>\n";
    print "<description><![CDATA[" . $SystemStatusDescr . "]]>\n</description>\n";
    print "</module>\n";

    #Resolve Raid Controller status
    $SrvStatusControllers = $$snmpresult{"$oidSrvStatusControllers"};
    #Print the Server ID
    print "<module>\n";
    print "<name>Fujitsu_RaidControllerStatus</name>\n";
    print "<type>generic_data</type>\n";
    print "<data><![CDATA[" . $SrvStatusControllers . "]]>\n</data>\n";
    print "<description><![CDATA[Fujitsu Server Raid Controller Status]]>\n</description>\n";
    print "</module>\n";

    #Resolve Raid Logical Drive status
    $SrvStatusLogicalDrives = $$snmpresult{"$oidSrvStatusLogicalDrives"};
    #Print the Server ID
    print "<module>\n";
    print "<name>Fujitsu_RaidLogicalDriveStatus</name>\n";
    print "<type>generic_data</type>\n";
    print "<data><![CDATA[" . $SrvStatusLogicalDrives . "]]>\n</data>\n";
    print "<description><![CDATA[Fujitsu Server Raid Logical Drive Status]]>\n</description>\n";
    print "</module>\n";

    #Resolve Raid Physical Drive status
    $SrvStatusPhysicalDrives = $$snmpresult{"$oidSrvStatusPhysicalDrives"};
    #Print the Server ID
    print "<module>\n";
    print "<name>Fujitsu_RaidPhysicalDriveStatus</name>\n";
    print "<type>generic_data</type>\n";
    print "<data><![CDATA[" . $SrvStatusPhysicalDrives . "]]>\n</data>\n";
    print "<description><![CDATA[Fujitsu Server Raid Physical Drive Status]]>\n</description>\n";
    print "</module>\n";

    #Resolve Raid Overall Drive status
    $SrvStatusStatusOverall = $$snmpresult{"$oidSrvStatusStatusOverall"};
    #Print the Server ID
    print "<module>\n";
    print "<name>Fujitsu_RaidOverallStatus</name>\n";
    print "<type>generic_data</type>\n";
    print "<data><![CDATA[" . $SrvStatusStatusOverall . "]]>\n</data>\n";
    print "<description><![CDATA[Fujitsu Server Raid Overall Status]]>\n</description>\n";
    print "</module>\n";

    #retrieve temperature OID's
    $snmptempresult = $session->get_entries( -columns => \@tempChecks );
    if ( !defined $snmptempresult ) {
        $message = $session->error;
        $session->close;
        print "SNMP-Agent returned \"$message\". (Is the Serverview Agent installed on the checked system?)\n";
        exit 3;
    }

    # store fetched temperature data
    foreach my $snmpkey ( keys %{$snmptempresult} ) {
     	push(@temps, $1) if ($snmpkey =~ m/$oidEnvTempStatus.(\d+\.\d+)/);
     }

    # sort fetched temperature data & print result
    @temps = Net::SNMP::oid_lex_sort(@temps);
    foreach my $tempID (@temps) {
 	my $tempStatus  = $$snmptempresult{$oidEnvTempStatus . '.' . $tempID};
 	next if ($tempStatus eq '99' or $tempStatus eq '2');
 	my $tempValue = $$snmptempresult{$oidEnvTempValue . '.' . $tempID};
 	my $tempWarn = $$snmptempresult{$oidEnvTempWarn . '.' . $tempID};
 	my $tempCrit = $$snmptempresult{$oidEnvTempCrit . '.' . $tempID};
 	my $tempDesc = $$snmptempresult{$oidEnvTempDesc . '.' . $tempID};
 	$tempDesc =~ s/[ ,;=]/_/g;

	print "<module>\n";
	print "<name>Fujitsu_". $tempDesc . "_TempValue</name>\n";
	print "<type>generic_data</type>\n";
	print "<data>" . $tempValue . "</data>\n";
	print "<description><![CDATA[". $tempDesc . " temperature in (C)]]>\n</description>\n";
	print "<min_warning>" . $tempWarn . "</min_warning>\n";
	print "<min_critical>" . $tempCrit . "</min_critical>\n"; 
	print "</module>\n";

	#  We evaluate the tempstatus and generate a "Pandora status"
	if ($tempValue <  $tempWarn)					#Ok status
	{
	    $PandoratempStatus = "1";
	}
	if (($tempValue >=  $tempWarn) && ( $tempValue < $tempCrit))	#Warning status
	{
	    $PandoratempStatus = "2";
	}
	if ($tempValue >  $tempCrit)					#Critical status
	{
	    $PandoratempStatus = "0";
	}

	print "<module>\n";
	print "<name>Fujitsu_". $tempDesc . "_Temp</name>\n";
	print "<type>generic_proc</type>\n";
	print "<data>" . $PandoratempStatus . "</data>\n";
	print "<description><![CDATA[The temperature of ". $tempDesc . "]]>\n</description>\n";
	print "</module>\n";

    }
   $snmpfanresult = $session->get_entries( -columns => \@fanChecks );
   if ( !defined $snmpfanresult ) {
        $message = $session->error;
        $session->close;
        print "SNMP-Agent returned \"$message\". (Is the Serverview Agent installed on the checked system?)\n";
        exit 3;
   }

	
   # store fetched fan data
   foreach my $snmpkey ( keys %{$snmpfanresult} ) {
	push(@fans, $1) if ($snmpkey =~ m/$oidEnvFanStatus.(\d+\.\d+)/);
   }
	
   # sort fetched fan data & print result
   @fans = Net::SNMP::oid_lex_sort(@fans);
   foreach $fanID (@fans) {

	$fanPurpose = $$snmpfanresult{$oidEnvFanPurpose . '.' . $fanID};
	$fanStatus  = $$snmpfanresult{$oidEnvFanStatus . '.' . $fanID};
	next if ($fanStatus eq '99' or $fanStatus eq '2' or $fanStatus eq '8');
	$fanSpeed = $$snmpfanresult{$oidEnvFanSpeed . '.' . $fanID};
	$fanDesc = $$snmpfanresult{$oidEnvFanDesc . '.' . $fanID};
	$fanDesc =~ s/\s//g;							#remove spaces from fanDesc
	$fanStatusDescr = $fanStatusDesc[$fanStatus];
	$fanPurposeDescr = $fanPurposeDesc[$fanPurpose];

	print "<module>\n";
	print "<name>Fujitsu_". $fanDesc . "_descr</name>\n";
	print "<type>generic_data</type>\n";
	print "<data>" . $fanSpeed . "</data>\n";
	print "<description><![CDATA[Fan status: ". $fanStatusDescr . " - The fan turns with a speed of " . $fanSpeed . " (rpm)]]>\n</description>\n";
	print "</module>\n";

	#  We evaluate the fanstatus and generate a "Pandora status"
	if ($fanStatus == 3 )						#Ok status
	{
	    $PandorafanStatus = "1";
	}
	else
	{
	    $PandorafanStatus = "0";					#All other statuses
	}

	print "<module>\n";
	print "<name>Fujitsu_". $fanDesc . "_status</name>\n";
	print "<type>generic_proc</type>\n";
	print "<data>" . $PandorafanStatus . "</data>\n";
	print "<description><![CDATA[". $fanDesc . " - " . $fanPurposeDescr . " :" . $fanStatusDescr . "]]>\n</description>\n";
	print "</module>\n";

	}

    #close SNMP-session
    $session->close;
}


	SWITCH: {
    if ( $ARGV[0] eq "--help" )    { print_help();     last SWITCH; }
    if ( $ARGV[0] eq "-h" )        { print_help();     last SWITCH; }
    if ( $ARGV[0] eq "--version" ) { print_revision(); last SWITCH; }
    if ( $ARGV[0] eq "-V" )        { print_revision(); last SWITCH; }
    if ( $ARGV[0] eq "-V" )        { print_revision(); last SWITCH; }
    if ( $#ARGV ne 1 ) { print_usage(); last SWITCH; }
    main();

}
